@@ -21,11 +21,7 @@ from enum import IntEnum
 
 from hypothesis.errors import Frozen, InvalidArgument
 from hypothesis.internal.compat import hbytes, hrange, text_type, \
-    int_to_bytes, benchmark_time, unicode_safe_repr
-
-
-def uniform(random, n):
-    return int_to_bytes(random.getrandbits(n * 8), n)
+    bit_length, benchmark_time, int_from_bytes, unicode_safe_repr
 
 
 class Status(IntEnum):
@@ -55,7 +51,7 @@ class ConjectureData(object):
         buffer = hbytes(buffer)
         return ConjectureData(
             max_length=len(buffer),
-            draw_bytes=lambda data, n, distribution:
+            draw_bytes=lambda data, n:
             hbytes(buffer[data.index:data.index + n])
         )
 
@@ -80,7 +76,8 @@ class ConjectureData(object):
         self.start_time = benchmark_time()
         self.events = set()
         self.forced_indices = set()
-        self.bind_points = set()
+        self.capped_indices = {}
+        self.interesting_origin = None
 
     def __assert_not_frozen(self, name):
         if self.frozen:
@@ -105,6 +102,8 @@ class ConjectureData(object):
     def draw(self, strategy):
         if self.depth >= MAX_DEPTH:
             self.mark_invalid()
+        if strategy.is_empty:
+            self.mark_invalid()
 
         if self.is_find and not strategy.supports_find:
             raise InvalidArgument((
@@ -118,26 +117,14 @@ class ConjectureData(object):
             if not self.frozen:
                 self.stop_example()
 
-    def mark_bind(self):
-        """Marks a point as somewhere that a bind occurs - that is, data
-        drawn after this point may depend significantly on data drawn prior
-        to this point.
-
-        Having points like this explicitly marked allows for better shrinking,
-        as we run a pass which tries to shrink the byte stream prior to a bind
-        point while rearranging what comes after somewhat to allow for more
-        flexibility. Trying that at every point in the data stream is
-        prohibitively expensive, but trying it at a couple dozen is basically
-        fine."""
-        self.bind_points.add(self.index)
-
     def start_example(self):
         self.__assert_not_frozen('start_example')
         self.interval_stack.append(self.index)
         self.level += 1
 
     def stop_example(self):
-        self.__assert_not_frozen('stop_example')
+        if self.frozen:
+            return
         self.level -= 1
         while self.level >= len(self.intervals_by_level):
             self.intervals_by_level.append([])
@@ -170,8 +157,25 @@ class ConjectureData(object):
         self.events = frozenset(self.events)
         del self._draw_bytes
 
-    def draw_bytes(self, n):
-        return self.draw_bytes_internal(n, uniform)
+    def draw_bits(self, n):
+        self.__assert_not_frozen('draw_bits')
+        if n == 0:
+            result = 0
+        elif n % 8 == 0:
+            return int_from_bytes(self.draw_bytes(n // 8))
+        else:
+            n_bytes = (n // 8) + 1
+            self.__check_capacity(n_bytes)
+            buf = bytearray(self._draw_bytes(self, n_bytes))
+            assert len(buf) == n_bytes
+            mask = (1 << (n % 8)) - 1
+            buf[0] &= mask
+            self.capped_indices[self.index] = mask
+            buf = hbytes(buf)
+            self.__write(buf)
+            result = int_from_bytes(buf)
+        assert bit_length(result) <= n
+        return result
 
     def write(self, string):
         self.__assert_not_frozen('write')
@@ -199,18 +203,19 @@ class ConjectureData(object):
         self.buffer.extend(result)
         self.intervals.append((initial, self.index))
 
-    def draw_bytes_internal(self, n, distribution):
+    def draw_bytes(self, n):
         self.__assert_not_frozen('draw_bytes')
         if n == 0:
             return hbytes(b'')
         self.__check_capacity(n)
-        result = self._draw_bytes(self, n, distribution)
+        result = self._draw_bytes(self, n)
         assert len(result) == n
         self.__write(result)
         return hbytes(result)
 
-    def mark_interesting(self):
+    def mark_interesting(self, interesting_origin=None):
         self.__assert_not_frozen('mark_interesting')
+        self.interesting_origin = interesting_origin
         self.status = Status.INTERESTING
         self.freeze()
         raise StopTest(self.testcounter)
