@@ -131,14 +131,15 @@ defines_strategy_with_reusable_values = base_defines_strategy(True)
 
 
 class Nothing(SearchStrategy):
-    @property
-    def is_empty(self):
+    def calc_is_empty(self, recur):
         return True
 
     def do_draw(self, data):
-        data.mark_invalid()
+        # This method should never be called because draw() will mark the
+        # data as invalid immediately because is_empty is True.
+        assert False  # pragma: no cover
 
-    def calc_has_reusable_values(self):
+    def calc_has_reusable_values(self, recur):
         return True
 
     def __repr__(self):
@@ -390,9 +391,6 @@ def tuples(*args):
     for arg in args:
         check_strategy(arg)
 
-    for arg in args:
-        if arg.is_empty:
-            return nothing()
     from hypothesis.searchstrategy.collections import TupleStrategy
     return TupleStrategy(args, tuple)
 
@@ -424,13 +422,13 @@ def lists(
     elements=None, min_size=None, average_size=None, max_size=None,
     unique_by=None, unique=False,
 ):
-    """Returns a list containing values drawn from elements length in the
+    """Returns a list containing values drawn from elements with length in the
     interval [min_size, max_size] (no bounds in that direction if these are
     None). If max_size is 0 then elements may be None and only the empty list
     will be drawn.
 
     average_size may be used as a size hint to roughly control the size
-    of list but it may not be the actual average of sizes you get, due
+    of the list but it may not be the actual average of sizes you get, due
     to a variety of factors.
 
     If unique is True (or something that evaluates to True), we compare direct
@@ -451,13 +449,6 @@ def lists(
         else:
             return builds(list)
     check_strategy(elements)
-    if elements.is_empty:
-        if (min_size or 0) > 0:
-            raise InvalidArgument((
-                'Cannot create non-empty lists with elements drawn from '
-                'strategy %r because it has no values.') % (elements,))
-        else:
-            return builds(list)
     if unique:
         if unique_by is not None:
             raise InvalidArgument((
@@ -490,21 +481,21 @@ def lists(
             min_size=min_size,
             key=unique_by
         )
-        return result
+    else:
+        from hypothesis.searchstrategy.collections import ListStrategy
+        if min_size is None:
+            min_size = 0
+        if average_size is None:
+            if max_size is None:
+                average_size = _AVERAGE_LIST_LENGTH
+            else:
+                average_size = (min_size + max_size) * 0.5
 
-    from hypothesis.searchstrategy.collections import ListStrategy
-    if min_size is None:
-        min_size = 0
-    if average_size is None:
-        if max_size is None:
-            average_size = _AVERAGE_LIST_LENGTH
-        else:
-            average_size = (min_size + max_size) * 0.5
-
-    return ListStrategy(
-        (elements,), average_length=average_size,
-        min_size=min_size, max_size=max_size,
-    )
+        result = ListStrategy(
+            (elements,), average_length=average_size,
+            min_size=min_size, max_size=max_size,
+        )
+    return result
 
 
 @cacheable
@@ -568,7 +559,7 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
 
 @defines_strategy
 def fixed_dictionaries(mapping):
-    """Generate a dictionary of the same type as mapping with a fixed set of
+    """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. mapping must be a dict subclass.
 
     Generated values have all keys present in mapping, with the
@@ -581,9 +572,6 @@ def fixed_dictionaries(mapping):
     check_type(dict, mapping, 'mapping')
     for v in mapping.values():
         check_strategy(v)
-    for v in mapping.values():
-        if v.is_empty:
-            return nothing()
     return FixedKeysDictStrategy(mapping)
 
 
@@ -650,7 +638,7 @@ def characters(whitelist_categories=None, blacklist_categories=None,
 
     If ``whitelist_categories`` is specified, then only characters from those
     Unicode categories will be produced. This is a further restriction,
-    characters must also satisfy `min_codepoint` and `max_codepoint`.
+    characters must also satisfy ``min_codepoint`` and ``max_codepoint``.
 
     If ``blacklist_categories`` is specified, then any character from those
     categories will not be produced. This is a further restriction,
@@ -750,16 +738,15 @@ def text(
 @defines_strategy
 def from_regex(regex):
     """Generates strings that contain a match for the given regex (i.e. ones
-    for which re.search will return a non-None result).
+    for which :func:`re.search` will return a non-None result).
 
     ``regex`` may be a pattern or :func:`compiled regex <python:re.compile>`.
     Both byte-strings and unicode strings are supported, and will generate
     examples of the same type.
 
-    You can use regex flags (such as :const:`python:re.IGNORECASE`,
-    :const:`python:re.DOTALL` or :const:`python:re.UNICODE`) to control
-    generation. Flags can be passed either in compiled regex or inside the
-    pattern with a ``(?iLmsux)`` group.
+    You can use regex flags such as :const:`re.IGNORECASE`, :const:`re.DOTALL`
+    or :const:`re.UNICODE` to control generation. Flags can be passed either
+    in compiled regex or inside the pattern with a ``(?iLmsux)`` group.
 
     Some regular expressions are only partly supported - the underlying
     strategy checks local matching and relies on filtering to resolve
@@ -1014,7 +1001,7 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
     If min_value is not None then all generated values are no less than
     min_value.  If max_value is not None then all generated values are no
     greater than max_value.  min_value and max_value may be anything accepted
-    by the `python:`~fractions.Fraction` constructor.
+    by the :class:`~fractions.Fraction` constructor.
 
     If max_denominator is not None then the denominator of any generated
     values is no greater than max_denominator. Note that max_denominator must
@@ -1106,7 +1093,7 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
 @defines_strategy_with_reusable_values
 def decimals(min_value=None, max_value=None,
              allow_nan=None, allow_infinity=None, places=None):
-    """Generates instances of decimals.Decimal, which may be:
+    """Generates instances of :class:`decimals.Decimal`, which may be:
 
     - A finite rational number, between ``min_value`` and ``max_value``.
     - Not a Number, if ``allow_nan`` is True.  None means "allow NaN, unless
@@ -1115,7 +1102,7 @@ def decimals(min_value=None, max_value=None,
       respectively are None, and ``allow_infinity`` is not False.  None means
       "allow infinity, unless excluded by the min and max values".
 
-    Note that where floats have one `NaN` value, Decimals have four: signed,
+    Note that where floats have one ``NaN`` value, Decimals have four: signed,
     and either *quiet* or *signalling*.  See `the decimal module docs
     <https://docs.python.org/3/library/decimal.html#special-values>`_ for
     more information on special values.
@@ -1206,14 +1193,14 @@ def recursive(base, extend, max_leaves=100):
     max_leaves: The maximum number of elements to be drawn from base on a given
     run.
 
-    This returns a strategy S such that S = extend(base | S). That is, values
-    maybe drawn from base, or from any strategy reachable by mixing
+    This returns a strategy ``S`` such that ``S = extend(base | S)``. That is,
+    values may be drawn from base, or from any strategy reachable by mixing
     applications of | and extend.
 
-    An example may clarify: recursive(booleans(), lists) would return a
+    An example may clarify: ``recursive(booleans(), lists)`` would return a
     strategy that may return arbitrarily nested and mixed lists of booleans.
-    So e.g. False, [True], [False, []], [[[[True]]]], are all valid values to
-    be drawn from that strategy.
+    So e.g. ``False``, ``[True]``, ``[False, []]``, and ``[[[[True]]]]`` are
+    all valid values to be drawn from that strategy.
 
     """
 
@@ -1224,7 +1211,7 @@ def recursive(base, extend, max_leaves=100):
 @defines_strategy
 def permutations(values):
     """Return a strategy which returns permutations of the collection
-    "values"."""
+    ``values``."""
     from hypothesis.internal.conjecture.utils import integer_range
 
     values = list(values)
@@ -1498,10 +1485,10 @@ def choices():
 @cacheable
 @defines_strategy_with_reusable_values
 def uuids():
-    """Returns a strategy that generates UUIDs.
+    """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
     All returned values from this will be unique, so e.g. if you do
-    lists(uuids()) the resulting list will never contain duplicates.
+    ``lists(uuids())`` the resulting list will never contain duplicates.
 
     """
     from uuid import UUID
@@ -1680,7 +1667,8 @@ def check_valid_bound(value, name):
 def try_convert(typ, value, name):
     if value is None:
         return None
-
+    if isinstance(value, typ):
+        return value
     try:
         return typ(value)
     except TypeError:
